/** 
 * Substitute for xeventx
 * */

const EventualContext = require('./context');
const EventParam = require('./event-param');


class Eventual {
	constructor() {
		this._events = {
			ctx: [],
			state: {},
			store: {}
		};
	}

	emit(name,...args) {
		// Special send to all eventual contexts
		var tocall = [];		
		this._events.ctx.forEach((c) => {
			var channel = c.get(name);
			if(channel)tocall.push(...channel.listeners);
		});
		return this.process(tocall,args)
	}

	// process should be handled here
	process(tocall,nargs) {
		var args = nargs || [];	 // or empty
		var evt = new EventParam(next,args);
		
		var callback,dotrigger = 0;
		//Inner trigger	
		// Finish	
		function triggerDone() {
			if(callback) callback(evt);
			dotrigger = 1;
		}
		var ret = {
			done: function(cb) {
				if(dotrigger) { cb(evt); }
				callback = cb;
			}
		}
		tocall = tocall.sort((a,b) => {
			return (a.order - b.order) 
		});	
		// Get iter and setup evt
		var iter = tocall[Symbol.iterator]();
		
		function next() {
			process.nextTick(() => chain(iter));
		}

		// Internal thing
		function chain(iter) { 
			// Stack overflow risk? maybe put this in process.tick();
			var v = iter.next();
			if(v.done) {
				triggerDone(); // Done callback with evt only
				return;
			}
			// Create evt here with some global event chaining iteraction	
			try {
				// Setup entry info
				var entry = v.value;
				evt.args = args;
				evt.name = entry.name;
				entry.callback(...args,evt);
				evt.count++;
			}catch(e) {
				console.log(e);
				evt.errors.push(e);
				// Should we stop??
			}
			if(!evt.dowait) next();
			evt.dowait=0; // reset wait?
		}
		next();

		return ret;
	}


	createContext() {
		var ret = new EventualContext(this);
		this._events.ctx.push(ret);
		return ret;
	}
	destroyContext(ctx) {
		var i = this._events.ctx.indexOf(ctx);
		if(i!=-1) this._events.ctx.splice(i,1);
	}
	state(name,args) {
		// Should do this per context
		this._events.state[name] = args;
	}


	
}

module.exports = Eventual;

