/** 
 * Substitute for xeventx
 * */

const Channel = require('./context/channel');
const EventualContext = require('./context');
const EventParam = require('./event-param');

class Eventual {
	constructor() {
		this._events = {
			ctx: new Set(),
			store: {}
		};
	}
	createContext() {
		var ret = new EventualContext(this);
		this._events.ctx.add(ret);
		return ret;
	}
	destroyContext(ctx) {
		this._events.ctx.delete(ctx);
	}

	set(name,value) {
		if(value == undefined) {
			delete this._events.store[name];
		}else {
			this._events.store[name] = value;
		}
		var tocall = [];	
		// Trigger value separatedaly
		this._events.ctx.forEach((c) => {
			var watchers = c.getWatchers(name);
			watchers.forEach((w) => {
				tocall.push(w);
			});
		});
		tocall.forEach((tc) => {
			tc(value);
		});
	}
	unset(name) {
		this.set(name,undefined);
	}

	// Private?
	emit(name,...args) {
		// Special send to all eventual contexts
		var tocall = this.callbacks(name);
		return this.process(tocall,args)
	}
	propagate(name,nameto,...args) {
		var nparts = name.split(Channel.SEP);
		var tocall = [];

		var sname = name;
		while(nparts.length && sname != nameto)  {
			var localCall = this.callbacks(sname);
			// Limitation on argument size
			tocall.push(...localCall);
			nparts.pop();
			sname = nparts.join(Channel.SEP);
		}
		var localCall = this.callbacks(sname);
		tocall.push(...localCall);
		return this.process(tocall,args);	
	}

	callbacks(name) {
		var tocall = [];		
		this._events.ctx.forEach((c) => {
			var callbacks = c.getCallbacks(name);
			callbacks.forEach((l) => { tocall.push(l); });
		});
		return tocall;
	}

	processsimple(tocall,nargs) {
		tocall.forEach((l) => {
			l.callback(...nargs);
		});
	}
	process(tocall,nargs) {
		var args = nargs || [];

		tocall = tocall.sort((a,b) => {
			return (a.order - b.order);
		});
		var iter = tocall[Symbol.iterator]();
		var ret = new Promise((resolve,reject) => {
			var evt = new EventParam(next,args);
			function next() {
				process.nextTick(()=> chain(iter));
			}
			function chain(iter) {
				var v = iter.next();
				if(v.done) { resolve(evt); return; }
				try {
					var entry = v.value;
					evt.args = args;
					evt.name = entry.name;
					entry.callback(...args,evt);
					evt.consume();
					if(evt._dostop == 1) {
						resolve(evt);
						return;	
					}
				} catch(e)  {
					console.log(e);
					evt.errors.push(e);
				}
				if(!evt._dowait) next();
				evt._dowait = 0;
			}
			next(); // Start chain
		});
		return ret;
	}
	// process should be handled here
	processRetDone(tocall,nargs) {
		var args = nargs || [];	 // or empty
		var evt = new EventParam(next,args); // Travel between events

		var callback,dotrigger = 0;
		function triggerDone() {
			if(callback) callback(evt);
			dotrigger = 1;
		}

		var ret = {
			done: function(cb) {
				if(dotrigger) { cb(evt); }
				callback = cb;
			}
		}
		tocall = tocall.sort((a,b) => {
			return (a.order - b.order) 
		});	
		// Get iter and setup evt
		var iter = tocall[Symbol.iterator]();
		
		function next() {
			// prevent stack overflowing by defering function
			process.nextTick(() => chain(iter))
		}

		// Internal thing
		function chain(iter) { 
			// Stack overflow risk? maybe put this in process.tick();
			var v = iter.next();
			if(v.done) {
				triggerDone(); // Done callback with evt only
				return;
			}
			// Create evt here with some global event chaining iteraction	
			try {
				// Setup entry info
				var entry = v.value;
				evt.args = args;
				evt.name = entry.name;
				entry.callback(...args,evt);

				if(evt._dostop == 1) {
					triggerDone();
					return;	
				}
				evt.consume();
			}catch(e) {
				console.log(e);
				evt.errors.push(e);
				// Should we stop??
			}
			if(!evt._dowait) next();
			evt._dowait=0; // reset wait?
		}
		next();

		return ret;
	}

	
}

module.exports = Eventual;

